# -*- coding: utf-8 -*-

import json
from typing import Dict, List, Any

import pandas as pd

from .country_code_profile import CountryCodeProfile
from .profile import ChannelIdTypeProfile


class FfromLoginProfile():
    """
    登录来源
        主要是不同android品牌特征，android和ios特征在其他画像中已有，这里不重复构建
    """
    def __init__(self) -> None:
        pass

    def convert(self, value):
        ret = [0, 0, 0, 0, 0]
        try:
            value = value.lower()
        except Exception as e:
            return ret

        if 'huawei' in value:
            ret[0] = 1
        elif 'vivo' in value:
            ret[1] = 1
        elif 'oppo' in value:
            ret[2] = 1
        elif 'xiaomi' in value:
            ret[3] = 1
        else:
            ret[4] = 1
        return ret


class UserPreferenceCateProfile():

    def __init__(self) -> None:
        cate_list = ['个人成长', '亲子教育', '人际关系', '婚姻家庭', '心理健康', '恋爱情感', '情绪压力', '职场发展']
        self.cate_index = {
            cate: index for index, cate in enumerate(cate_list)
        }

    def convert(self, value):
        ret = [0.] * 8
        if pd.isnull(value):
            return ret
        
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except Exception as e:
                return ret

        for info in value:
            ret[self.cate_index[info['cate_name']]] = info['preference_score']

        return ret


class NumClassProfile():

    def __init__(self, split_points, mode='le') -> None:
        """
        mode : le 前开后闭； be: 前闭后开；默认前开后闭
        """
        self.split_points = split_points
        self.mode = mode

    def value_index(self, value):
        for i, v in enumerate(self.split_points):
            if self.mode == 'be':
                if value < v:
                    return i
            else:
                if value <= v:
                    return i
        return len(self.split_points)

    def convert(self, value):
        ret = [0] * (len(self.split_points) + 1)
        if pd.isnull(value):
            return ret
        try:
            value = float(value)
            index = self.value_index(value)
            ret[index] = 1
        except:
            return ret
        return ret


class AidiCstBiasPriceProfile():

    def convert(self, value):
        ret = [0] * 6
        if pd.isnull(value):
            return ret

        for v in value:
            try:
                ret[v['level'] - 1] = 1
            except Exception:
                pass
        return ret


class MultiChoiceProfile():

    def __init__(self, option_dict: Dict[Any, int]) -> None:
        self.option_dict = option_dict

    def convert(self, value: List):
        ret = [0] * len(self.option_dict)
        if pd.isnull(value):
            return ret

        for v in value:
            try:
                i = self.option_dict[v]
                ret[i] = 1
            except Exception as e:
                pass
        return ret


class CityProfile():

    def __init__(self) -> None:
        self.default_city_codes = [
            '330500', '640200', '130200', '620200', '321000', '530600', '650500', '410200', '511400', '450200',
            '610800', '220400', '430400', '320500', '410400', '341100', '420300', '410500', '640100', '440100',
            '420500', '650200', '441900', '211200', '210400', '140700', '131000', '440700', '340100', '350200',
            '371100', '370900', '130500', '451300', '331100', '320700', '710100', '500000', '610300', '370100',
            '610500', '450300', '520100', '140200', '320400', '210500', '440300', '610700', '341800', '210300',
            '340200', '120100', '340500', '210200', '222400', '370600', '110100', '441200', '230500', '510100',
            '330700', '330600', '370300', '230600', '450100', '340300', '651800', '340800', '430200', '421300',
            '220600', '150200', '433100', '440500', '620100', '710200', '130100', '131100', '150600', '430100',
            '150100', '130400', '140600', '140300', '410300', '620600', '330300', '321100', '320900', '630100',
            '320100', '410900', '510400', '620800', '610600', '220300', '420600', '510700', '130300', '411400',
            '310000', '341200', '370500', '710500', '231100', '152900', '371500', '220100', '360700', '150500',
            '331000', '360600', '371000', '341600', '130600', '230100', '410800', '370700', '410700', '430800',
            '410100', '210800', '330400', '460200', '650100', '310100', '350500', '360400', '320300', '500100',
            '360900', '610100', '350100', '350400', '530100', '320600', '130900', '371300', '421200', '210700',
            '220200', '130700', '320800', '420100', '110000', '150400', '442000', '469002', '360100', '150800',
            '441300', '460100', '610200', '210100', '210900', '371400', '621000', '141000', '330100', '220700',
            '371700', '370800', '211400', '330200', '140400', '120000', '231200', '140100', '431100', '320200',
            '451000', '370200', '511900', '361100', '610400', '440600', '411100', '231000', '360300'
        ]
        self.city_codes = self.default_city_codes
        self.city_code_dict = {
            code: index for index, code in enumerate(self.city_codes)
        }

    def convert(self, value):
        ret = [0] * len(self.city_code_dict)
        if pd.isnull(value):
            return ret

        value = str(value)
        try:
            i = self.city_code_dict[value]
            ret[i] = 1
        except Exception as e:
            pass
        return ret


class AidiCstBiasCityProfile(CityProfile):

    def __init__(self) -> None:
        super().__init__()

    def convert(self, value):
        ret = [0] * len(self.city_code_dict)

        if pd.isnull(value):
            return ret

        if not value:
            return ret

        if isinstance(value, str):
            try:
                value = json.loads(value)
            except Exception as e:
                pass
        
        for v in value:
            try:
                ret[self.city_code_dict[v]] = 1
            except Exception as e:
                pass
        return ret


profile_converters = [
    ['country_code', CountryCodeProfile()],
    ['channel_id_type', ChannelIdTypeProfile()],
    ['ffrom_login', FfromLoginProfile()],
    ['user_preference_cate', UserPreferenceCateProfile()],
    ['consult_pay_money', NumClassProfile([100, 300, 1000])],
    ['listen_pay_money', NumClassProfile([0, 50])],
    ['test_items_pay_money', NumClassProfile([0, 10])],
    ['course_pay_money', NumClassProfile([0])],
    ['consult_order_num', NumClassProfile([0, 1])],
    ['listen_order_num', NumClassProfile([0])],
    ['test_items_order_num', NumClassProfile([0])],
    ['course_order_num', NumClassProfile([0])],
    ['aidi_cst_bias_city', AidiCstBiasCityProfile()],
    ['aidi_cst_bias_sex', MultiChoiceProfile({i + 1: i for i in range(2)})],
    ['aidi_cst_bias_price', AidiCstBiasPriceProfile()],
    ['aidi_cst_bias_server_type', MultiChoiceProfile({i + 1: i for i in range(4)})],
    ['user_login_city', CityProfile()],
    ['d30_inquire_order_num', NumClassProfile([0, 1])],
    ['d30_session_num', NumClassProfile([0, 1])],
]

